
setwd("D:/neuroimaging/R code for statmod/")
locations2D <- read.table("sensor coords.txt")      # load sensor 2Dcoords
coords <- as.matrix(locations2D[,2:3])
source("functions-statmod.R")  # load functions for dipole modelling

# load one single trial data (the data used here correspond to those used in section 6.1)
nsensors <- 248 # num. sensors
ntimes <- 256   # num. of observations taken over a time window of 1 sec (-0.5:0.5 secs); one obs each 4 msecs
signal.scale.factor <- 10e+12   # scale factor to apply to the MEG measurements
signal <- t(as.matrix(read.table("data.txt", header = T), nrow = nsensors, ncol=ntimes, byrow = T))
y <- signal*signal.scale.factor
timescale <- seq(-0.5, 0, length.out=ntimes/2)   # select only the pre-stimulus time window (from -0.5 to 0 secs)



### Analysis of a single trial

# fit the dipole model
library(spam)
fit <- dipolefitMEG(y, coords, pre.stimulus=T, edf.target=60, df.mean.gamma=7, df.amplitude=3, df.phase=3)
  # y: MEG data in a matrix of dim S x T (S=nsensors, T=ntimes)
  # coords: sensor x-y coordinates in a matrix of dim S x 2
  # pre.stimulus=T indicates the dipole is searched over the pre-stimulus time
  # edf.target: degrees of freedom for the smooth pole signals (gamma)
  # df.mean.gamma: degrees of freedom for the smooth intercept curve (delta)
  # df.amplitude: degrees of freedom for the smooth amplitude curve (alpha)
  # df.phase: degrees of freedom for the smooth phase curve (phi)



# plot Fig 3: brainmap, data, estimated pole signals and their quasiperiodic decomposition 

# panel a: the fitted dipole topography at the peak
n.levels <- pretty(range(c(fit$y.hat)/signal.scale.factor, na.rm=T), 200)
cols <- rev(rainbow(length(n.levels) - 1))
ind.peak.tim <- 42
library(akima)
library(fields)
plotMEG.brainImage(fit$y.hat[,ind.peak.tim]/signal.scale.factor, coords, sensordots =F, sensorlabels = F, barplot=T, writetitle = "dipole topography", cols = cols, breaks = n.levels) 
x.pole1 <- fit$loc.hat[1] + fit$r.hat*cos(fit$orientation.hat)
y.pole1 <- fit$loc.hat[2] + fit$r.hat*sin(fit$orientation.hat)
x.pole2 <- fit$loc.hat[1] - fit$r.hat*cos(fit$orientation.hat) 
y.pole2 <- fit$loc.hat[2] - fit$r.hat*sin(fit$orientation.hat)
ind1 <- which( sqrt((coords[,1]-x.pole1)^2 + (coords[,2]-y.pole1)^2) < (fit$h.hat*1) )
ind2 <- which( sqrt((coords[,1]-x.pole2)^2 + (coords[,2]-y.pole2)^2) < (fit$h.hat*1) )
ind1.toplot <- setdiff(ind1,ind2)
ind2.toplot <- setdiff(ind2,ind1)
points(coords[ind1.toplot,1], coords[ind1.toplot,2], pch=16)
points(coords[ind2.toplot,1], coords[ind2.toplot,2], pch=16, col="orange")
draw.orientation(fit$loc.hat[1], fit$loc.hat[2], fit$orientation.hat, length.line = 4, color = "black", line.width = 1)

# panel b: MEG data
ylim0 <- range(y/signal.scale.factor, na.rm=T)
par(mar = c(5, 5, 4, 2) + 0.1)
plot(timescale, y[ind2.toplot[1],1:128]/signal.scale.factor, type="l", col="orange", 
     ylim=ylim0, main="MEG data",
     cex.lab = 1.6, ylab="noisy data (Tesla)", xlab="time (s)")
for(i in ind1.toplot) lines(timescale, y[i,1:128]/signal.scale.factor, col=1)
for(i in ind2.toplot) lines(timescale, y[i,1:128]/signal.scale.factor, col="orange")


# panel c: the fitted pole signals
par(mar = c(5, 5, 4, 2) + 0.1)
plot(timescale, (fit$qp.pole1)$y.qp.decomposition/signal.scale.factor, type="l", col="black", ylim=ylim0, 
     cex.lab = 1.6, ylab="dipole oscillation (Tesla)", xlab="time (s)", lwd=2, main="pole signals")
lines(timescale, (fit$qp.pole2)$y.qp.decomposition/signal.scale.factor, col="orange", lwd=2)
lines(timescale, (fit$y.pole1.hat)/signal.scale.factor, col="black", lty=2, lwd=1.5)
lines(timescale, (fit$y.pole2.hat)/signal.scale.factor, col="orange", lty=2, lwd=1.5)

# panel d: the smooth intercept of each pole signal
plot(fit$qp.pole1$intercept.hat/signal.scale.factor, col=1, main="amplitude", 
      ylim=ylim0, ylab="mean (Tesla)", xlab="time (s)", type="l")
lines(fit$qp.pole2$intercept.hat/signal.scale.factor, col="orange")


# panel e: the amplitude curve of each pole signal
plot(fit$qp.pole1$amplitude.hat/signal.scale.factor, col=1, main="amplitude", 
     ylim=ylim0, ylab="amplitude (Tesla)", xlab="time (s)", type="l")
lines(fit$qp.pole2$amplitude.hat/signal.scale.factor, col="orange")

# panel f: the phase curve of each pole signal
plot(fit$qp.pole1$phase.hat, col=1, main="phase", type="l", ylim=c(0,40),
     , ylab="phase (radian)", xlab="time (s)")
lines(fit$qp.pole2$phase.hat, col="orange")

# panel g: the frequency curve of each pole signal
plot(fit$qp.pole1$freq.hat, col=1, main="frequency", type="l", ylim=c(0,30),
     ,ylab="frequecy (Hz)", xlab="time (s)")
lines(fit$qp.pole2$freq.hat, col="orange")

# the out of phase information, see formula (8) in the paper
plot(timescale, fit$out.of.phase, main="classic")

# strength of evidence for a dipole expressed as a curve over time (see w(t) in section 6.2 in the paper)
plot(timescale[-128], fit$dipole.strength, main="classic")


# dynamic plot of fitted values using rpanel R package
library(rpanel)
library(tkrplot)
n.levels <- pretty(range(fit$y.hat,na.rm = T), 50)
cols <- rev(rainbow(length(n.levels) - 1))

redraw.plot <- function(panel) {
  rp.tkrreplot(panel, plot.panel)
  panel
}
panel.draw <- function(panel){
  i <- 1 + (panel$Time - min(panel$timescale)) / diff(panel$timescale)[1]
  i <- min(i, length(panel$timescale))
  i <- round(i,0)
  plotMEG.brainImage(panel$meg[,i], panel$coords, sensordots =T, sensorlabels = F, barplot=F, 
                     writetitle = panel$title.plot, cols = panel$cols, breaks = panel$n.levels)
  panel
}
panel <- rp.control(meg = fit$y.hat, timescale = timescale, Time = timescale[1], coords = as.matrix(locations2D[,2:3], ncol = 2), 
                    title.plot = "fit", n.levels=n.levels, cols=cols)
rp.tkrplot(panel, plot.panel, panel.draw, hscale = 1.6, vscale = 1.6, row = 0, column = 0)
rp.slider(panel, Time, timescale[1], timescale[length(timescale)], redraw.plot, showvalue = TRUE, row = 1, column = 0, columnspan = 2, sticky = "ew")   

